%{ open Ast %}

%token FOR CONST MATRIX POINT POINTS FUNC LITERAL
%token SLEEP RENDER PLUSEQUALS FROM TO INT_LITERAL
%token LBRACE RBRACE LBRACKET RBRACKET ARROW
%token SEMI LPAREN RPAREN LBRACE RBRACE COMMA EOF
%token PLUS MINUS TIMES DIVIDE ASSIGN POWER

%token <string> LITERAL
%token <string> INT_LITERAL
%token <string> ID

%right ASSIGN
%left PLUS MINUS
%left TIMES DIVIDE
%left POWER
%nonassoc ID

%start program
%type <Ast.program> program

%%

program:
  stmt_list { $1 }

stmt_list:
    /* nothing */  { [] }
  | stmt stmt_list { $1 :: $2 }

stmt:
    def SEMI { Def($1) }
  | SLEEP expr SEMI { Sleep($2) }
  | RENDER ID ID expr SEMI { Render($2, $3, $4) }
  | ID PLUSEQUALS point SEMI { AddPoint($1, $3) }
  | FOR ID FROM INT_LITERAL TO INT_LITERAL LBRACE stmt_list RBRACE
          { ForLoop($2, $4, $6, $8) }

def:
    CONST ID ASSIGN expr { ConstDef($2, $4) }
  | FUNC ID ASSIGN ID ARROW expr { FuncDef($2, $4, $6) }
  | POINT ID ASSIGN point { PointDef($2, $4) }
  | POINTS ID ASSIGN LBRACKET point_list RBRACKET { PointsDef($2, List.rev $5) }
  | MATRIX ID ASSIGN matrix { MatrixDef($2, $4) }

point:
    LBRACE expr COMMA expr COMMA expr RBRACE { PointLit({x=$2; y=$4; z=$6}) }
  | ID { PointId($1) }

matrix:
    LBRACKET matrix_entries RBRACKET { MatrixLit(List.rev $2) }
  | matrix_chain { MatrixChain(List.rev $1) }

matrix_chain:
    /* nothing */  { [] }
  | ID { [$1] }
  | matrix_chain TIMES ID { $3 :: $1 }

point_list:
    /* nothing */  { [] }
  | point { [$1] }
  | point_list COMMA point { $3 :: $1 }

matrix_entries:
    /* nothing */  { [] }
  | expr { [$1] }
  | matrix_entries COMMA expr { $3 :: $1 }

expr:
    expr PLUS expr { Binop($1, Add, $3) }
  | expr MINUS expr { Binop($1, Sub, $3) }
  | expr TIMES expr { Binop($1, Mul, $3) }
  | expr DIVIDE expr { Binop($1, Div, $3) }
  | expr POWER expr { Binop($1, Pow, $3) }
  | ID { Id($1) }
  | LITERAL { Lit($1) }
  | INT_LITERAL { Lit($1) }
  | LPAREN expr RPAREN { $2 }
  | ID expr { FuncApplication($1, $2) }
