{ open Parser }

let digit = ['0'-'9']
let digits = (digit)+
let fractional = '.'(digits)

let letter = ['a'-'z''A'-'Z''_']
let letter_or_digit = (letter) | (digit)

rule token =
  parse [' ' '\t' '\r' '\n'] { token lexbuf }
    | '+'   { PLUS }
    | "+="  { PLUSEQUALS }
    | '-'   { MINUS }
    | '*'   { TIMES }
    | "**"  { POWER }
    | '/'   { DIVIDE }
    | '('   { LPAREN }
    | ')'   { RPAREN }
    | '{'   { LBRACE }
    | '}'   { RBRACE }
    | '['   { LBRACKET }
    | ']'   { RBRACKET }
    | ';'   { SEMI }
    | ','   { COMMA }
    | '='   { ASSIGN }
    | "->"  { ARROW }
    | "for"      { FOR }
    | "from"     { FROM }
    | "to"      { TO }
    | "const"    { CONST }
    | "matrix"   { MATRIX }
    | "point"    { POINT }
    | "points"   { POINTS }
    | "func"     { FUNC }
    | "render"   { RENDER }
    | "sleep"    { SLEEP }
    | digits as int_lit { INT_LITERAL(int_lit) }
    | (digits)(fractional)? | (digits)?(fractional) as lit { LITERAL(lit) }
    | (letter)(letter_or_digit)* as identifier { ID(identifier) }
    | eof { EOF }
